// created by jay 0.7 (c) 1998 Axel.Schreiner@informatik.uni-osnabrueck.de

#line 2 "Parser.jay"
// -*-csharp-*-
// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Leonardo Mezzina, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using System.IO;
using System.Text;
using System.Collections;
using BoPi.Types;
using BoPi.Common;

namespace BoPi.Compiler{

// using VoidExpr = BoPi.Common.VoidExpr<ISymbolTableValueEntry>;
// using LocationExpr = BoPi.Compiler.LocationExpr<ISymbolTableValueEntry>;

public class BoPiParser {

  Lexer lexer;
  Hashtable locationMap;
  static public int yacc_verbose_flag = 2;

	// static IExpression mk(IExpression expr, int location)
	// { return new LocationExpr(expr, location); }

#line default

  /** error output stream.
      It should be changeable.
    */
  public System.IO.TextWriter ErrorOutput = System.Console.Out;

  /** simplified error message.
      @see <a href="#yyerror(java.lang.String, java.lang.String[])">yyerror</a>
    */
  public void yyerror (string message) {
    yyerror(message, null);
  }

  /** (syntax) error message.
      Can be overwritten to control message format.
      @param message text to be displayed.
      @param expected vector of acceptable tokens, if available.
    */
  public void yyerror (string message, string[] expected) {
    if ((yacc_verbose_flag > 0) && (expected != null) && (expected.Length  > 0)) {
      ErrorOutput.Write (message+", expecting");
      for (int n = 0; n < expected.Length; ++ n)
        ErrorOutput.Write (" "+expected[n]);
        ErrorOutput.WriteLine ();
    } else
      ErrorOutput.WriteLine (message);
  }

  /** debugging support, requires the package jay.yydebug.
      Set to null to suppress debugging messages.
    */
//t  internal yydebug.yyDebug debug;

  protected static  int yyFinal = 11;
//t  public static  string [] yyRule = {
//t    "$accept : process",
//t    "process : CHANOUT LPAREN E RPAREN",
//t    "process : CHANIN TIMES LPAREN F RPAREN DOT process",
//t    "process : CHANIN LPAREN F RPAREN DOT process",
//t    "process : SCHEMADEF schema_decl_ne_list IN process",
//t    "process : NEW ID COLON LT S GT K IN process",
//t    "process : NEW ID COLON LT S GT K LOCATION IN process",
//t    "process : NEW ID COLON LT S GT IN process",
//t    "process : NEW ID COLON LT S GT LOCATION IN process",
//t    "process : IMPORT ID COLON LT S GT K IN process",
//t    "process : IMPORT ID COLON LT S GT K IMPORTBODY IN process",
//t    "process : NEW error IN process",
//t    "process : SPAWN LBRACE process RBRACE process",
//t    "process : MATCH E WITH LBRACE Matchdec RBRACE",
//t    "process : JOIN_KW star_opt join_pattern_row_ne_list",
//t    "process : ZERO",
//t    "process : error",
//t    "star_opt :",
//t    "star_opt : TIMES",
//t    "join_pattern_row_ne_list : join_pattern_row",
//t    "join_pattern_row_ne_list : join_pattern_row_ne_list ORMATCH join_pattern_row",
//t    "join_pattern_row : join_pattern_atom_ne_list DOT process",
//t    "join_pattern_atom_ne_list : join_pattern_atom",
//t    "join_pattern_atom_ne_list : join_pattern_atom_ne_list AMPERSAND join_pattern_atom",
//t    "join_pattern_atom : CHANIN LPAREN F RPAREN",
//t    "schema_decl_ne_list : schema_decl",
//t    "schema_decl_ne_list : schema_decl_ne_list AND schema_decl",
//t    "schema_decl : ID EQ S",
//t    "Matchdec : F ARROW process",
//t    "Matchdec : Matchdec ORMATCH F ARROW process",
//t    "Matchdec : F ARROW process error",
//t    "F : S",
//t    "F : ID COLON F",
//t    "F : L LBRACK F RBRACK",
//t    "F : L LBRACK F RBRACK COMMA F",
//t    "E : LPAREN E RPAREN",
//t    "E : VOID",
//t    "E : ID",
//t    "E : NUMBER",
//t    "E : ZERO",
//t    "E : TEXT",
//t    "E : ID LBRACK E RBRACK",
//t    "E : ID LBRACK RBRACK",
//t    "E : E PLUS E",
//t    "E : E MINUS E",
//t    "E : E TIMES E",
//t    "E : E DIVIDE E",
//t    "E : E EQEQ E",
//t    "E : E NE E",
//t    "E : E GT E",
//t    "E : E GE E",
//t    "E : E LT E",
//t    "E : E LE E",
//t    "E : E OR E",
//t    "E : E AND E",
//t    "E : ID LBRACK E RBRACK COMMA E",
//t    "E : ID LBRACK RBRACK COMMA E",
//t    "E : NOT E",
//t    "E : MINUS E",
//t    "E : REST E",
//t    "E : FIRST E",
//t    "E : error",
//t    "L : ID",
//t    "L : TILDE",
//t    "L : L SHARP L",
//t    "L : L BACKSLASH L",
//t    "L : LBRACE L RBRACE",
//t    "S : LPAREN RPAREN",
//t    "S : VOID",
//t    "S : LPAREN S RPAREN",
//t    "S : S PLUS S",
//t    "S : ID",
//t    "S : L LBRACK S RBRACK",
//t    "S : L LBRACK RBRACK",
//t    "S : L LBRACK S RBRACK COMMA S",
//t    "S : L LBRACK RBRACK COMMA S",
//t    "S : LT S GT K",
//t    "S : T",
//t    "S : error",
//t    "K : I",
//t    "K : O",
//t    "K : IO",
//t    "K : error",
//t    "T : INT",
//t    "T : STRING",
//t    "T : ZERO",
//t    "T : NUMBER",
//t    "T : MINUS NUMBER",
//t    "T : TEXT",
//t  };
  protected static  string [] yyNames = {    
    "end-of-file",null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,null,null,null,null,null,null,null,
    null,null,null,null,null,null,null,"ERROR","ARROW","EQEQ","I","O",
    "IO","NE","LBRACE","RBRACE","LPAREN","RPAREN","LBRACK","RBRACK",
    "COLON","DOT","PLUS","MINUS","TIMES","DIVIDE","BACKSLASH","AMPERSAND",
    "EQ","LT","LE","GT","GE","COMMA","SHARP","TILDE","SPAWN","MATCH",
    "SELECT","INT","STRING","NEW","IN","VOID","WITH","SCHEMADEF","TEXT",
    "NUMBER","ZERO","CHANOUT","CHANIN","ID","EXP","OR","AND","NOT",
    "FIRST","REST","ORMATCH","LOCATION","IMPORT","IMPORTBODY","JOIN_KW",
    "Selectdec","Declaration","ORPLUS","UMINUS",
  };

  /** index-checked interface to yyNames[].
      @param token single character or %token value.
      @return token name or [illegal] or [unknown].
    */
  public static string yyname (int token) {
    if ((token < 0) || (token > yyNames.Length)) return "[illegal]";
    string name;
    if ((name = yyNames[token]) != null) return name;
    return "[unknown]";
  }

  /** computes list of expected tokens on error by tracing the tables.
      @param state for which to compute the list.
      @return list of token names.
    */
  protected string[] yyExpecting (int state) {
    int token, n, len = 0;
    bool[] ok = new bool[yyNames.Length];

    if ((n = yySindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyNames.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyNames[token] != null) {
          ++ len;
          ok[token] = true;
        }
    if ((n = yyRindex[state]) != 0)
      for (token = n < 0 ? -n : 0;
           (token < yyNames.Length) && (n+token < yyTable.Length); ++ token)
        if (yyCheck[n+token] == token && !ok[token] && yyNames[token] != null) {
          ++ len;
          ok[token] = true;
        }

    string [] result = new string[len];
    for (n = token = 0; n < len;  ++ token)
      if (ok[token]) result[n++] = yyNames[token];
    return result;
  }

  /** the generated parser, with debugging messages.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @param yydebug debug message writer implementing yyDebug, or null.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  internal Object yyparse (yyParser.yyInput yyLex, Object yyd)
				 {
//t    this.debug = (yydebug.yyDebug)yyd;
    return yyparse(yyLex);
  }

  /** initial size and increment of the state/value stack [default 256].
      This is not final so that it can be overwritten outside of invocations
      of yyparse().
    */
  protected int yyMax;

  /** executed at the beginning of a reduce action.
      Used as $$ = yyDefault($1), prior to the user-specified action, if any.
      Can be overwritten to provide deep copy, etc.
      @param first value for $1, or null.
      @return first.
    */
  protected Object yyDefault (Object first) {
    return first;
  }

  /** the generated parser.
      Maintains a state and a value stack, currently with fixed maximum size.
      @param yyLex scanner.
      @return result of the last reduction, if any.
      @throws yyException on irrecoverable parse error.
    */
  internal Object yyparse (yyParser.yyInput yyLex)
				{
    if (yyMax <= 0) yyMax = 256;			// initial size
    int yyState = 0;                                   // state stack ptr
    int [] yyStates = new int[yyMax];	                // state stack 
    Object yyVal = null;                               // value stack ptr
    Object [] yyVals = new Object[yyMax];	        // value stack
    int yyToken = -1;					// current input
    int yyErrorFlag = 0;				// #tks to shift

    int yyTop = 0;
    goto skip;
    yyLoop:
    yyTop++;
    skip:
    for (;; ++ yyTop) {
      if (yyTop >= yyStates.Length) {			// dynamically increase
        int[] i = new int[yyStates.Length+yyMax];
        yyStates.CopyTo (i, 0);
        yyStates = i;
        Object[] o = new Object[yyVals.Length+yyMax];
        yyVals.CopyTo (o, 0);
        yyVals = o;
      }
      yyStates[yyTop] = yyState;
      yyVals[yyTop] = yyVal;
//t      if (debug != null) debug.push(yyState, yyVal);

      yyDiscarded: for (;;) {	// discarding a token does not change stack
        int yyN;
        if ((yyN = yyDefRed[yyState]) == 0) {	// else [default] reduce (yyN)
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
//t            if (debug != null)
//t              debug.lex(yyState, yyToken, yyname(yyToken), yyLex.value());
          }
          if ((yyN = yySindex[yyState]) != 0 && ((yyN += yyToken) >= 0)
              && (yyN < yyTable.Length) && (yyCheck[yyN] == yyToken)) {
//t            if (debug != null)
//t              debug.shift(yyState, yyTable[yyN], yyErrorFlag-1);
            yyState = yyTable[yyN];		// shift to yyN
            yyVal = yyLex.value();
            yyToken = -1;
            if (yyErrorFlag > 0) -- yyErrorFlag;
            goto yyLoop;
          }
          if ((yyN = yyRindex[yyState]) != 0 && (yyN += yyToken) >= 0
              && yyN < yyTable.Length && yyCheck[yyN] == yyToken)
            yyN = yyTable[yyN];			// reduce (yyN)
          else
            switch (yyErrorFlag) {
  
            case 0:
              yyerror(String.Format ("syntax error, got token `{0}'", yyname (yyToken)), yyExpecting(yyState));
//t              if (debug != null) debug.error("syntax error");
              goto case 1;
            case 1: case 2:
              yyErrorFlag = 3;
              do {
                if ((yyN = yySindex[yyStates[yyTop]]) != 0
                    && (yyN += Token.yyErrorCode) >= 0 && yyN < yyTable.Length
                    && yyCheck[yyN] == Token.yyErrorCode) {
//t                  if (debug != null)
//t                    debug.shift(yyStates[yyTop], yyTable[yyN], 3);
                  yyState = yyTable[yyN];
                  yyVal = yyLex.value();
                  goto yyLoop;
                }
//t                if (debug != null) debug.pop(yyStates[yyTop]);
              } while (-- yyTop >= 0);
//t              if (debug != null) debug.reject();
              throw new yyParser.yyException("irrecoverable syntax error");
  
            case 3:
              if (yyToken == 0) {
//t                if (debug != null) debug.reject();
                throw new yyParser.yyException("irrecoverable syntax error at end-of-file");
              }
//t              if (debug != null)
//t                debug.discard(yyState, yyToken, yyname(yyToken),
//t  							yyLex.value());
              yyToken = -1;
              goto yyDiscarded;		// leave stack alone
            }
        }
        int yyV = yyTop + 1-yyLen[yyN];
//t        if (debug != null)
//t          debug.reduce(yyState, yyStates[yyV-1], yyN, yyRule[yyN], yyLen[yyN]);
        yyVal = yyDefault(yyV > yyTop ? null : yyVals[yyV]);
        switch (yyN) {
case 1:
#line 128 "Parser.jay"
  { yyVal = new Output (((StringValue)yyVals[-3+yyTop]).Value, ((IExpression)yyVals[-1+yyTop]), ((StringValue)yyVals[-3+yyTop]).Location); }
  break;
case 2:
#line 131 "Parser.jay"
  { yyVal = new Input (((StringValue)yyVals[-6+yyTop]).Value, true, ((IPattern)yyVals[-3+yyTop]), ((IProcess)yyVals[0+yyTop]), ((StringValue)yyVals[-6+yyTop]).Location); }
  break;
case 3:
#line 134 "Parser.jay"
  { yyVal = new Input (((StringValue)yyVals[-5+yyTop]).Value, false, ((IPattern)yyVals[-3+yyTop]), ((IProcess)yyVals[0+yyTop]), ((StringValue)yyVals[-5+yyTop]).Location); }
  break;
case 4:
#line 137 "Parser.jay"
  { yyVal = new SchemaDec(((ArrayList)yyVals[-2+yyTop]), ((IProcess)yyVals[0+yyTop]), ((int)yyVals[-3+yyTop])); }
  break;
case 5:
#line 140 "Parser.jay"
  { yyVal = new New(((StringValue)yyVals[-7+yyTop]).Value, new Chan(((IType)yyVals[-4+yyTop]), ((int)yyVals[-2+yyTop])), ((IProcess)yyVals[0+yyTop]), "", ((int)yyVals[-8+yyTop])); }
  break;
case 6:
#line 143 "Parser.jay"
  { yyVal = new New(((StringValue)yyVals[-8+yyTop]).Value, new Chan(((IType)yyVals[-5+yyTop]), ((int)yyVals[-3+yyTop])), ((IProcess)yyVals[0+yyTop]), ((string)yyVals[-2+yyTop]), ((int)yyVals[-9+yyTop])); }
  break;
case 7:
#line 146 "Parser.jay"
  { yyVal = new New(((StringValue)yyVals[-6+yyTop]).Value, new Chan(((IType)yyVals[-3+yyTop]), Chan.CAPABILITY.OUT), ((IProcess)yyVals[0+yyTop]), "", ((int)yyVals[-7+yyTop])); }
  break;
case 8:
#line 149 "Parser.jay"
  { yyVal = new New(((StringValue)yyVals[-7+yyTop]).Value, new Chan(((IType)yyVals[-4+yyTop]), Chan.CAPABILITY.OUT), ((IProcess)yyVals[0+yyTop]), ((string)yyVals[-2+yyTop]), ((int)yyVals[-8+yyTop])); }
  break;
case 9:
#line 152 "Parser.jay"
  { yyVal = new ImportDec(((StringValue)yyVals[-7+yyTop]).Value, new Chan(((IType)yyVals[-4+yyTop]), ((int)yyVals[-2+yyTop])), ((IProcess)yyVals[0+yyTop]), "", ((int)yyVals[-8+yyTop])); }
  break;
case 10:
#line 155 "Parser.jay"
  { yyVal = new ImportDec(((StringValue)yyVals[-8+yyTop]).Value, new Chan(((IType)yyVals[-5+yyTop]), ((int)yyVals[-3+yyTop])), ((IProcess)yyVals[0+yyTop]), ((string)yyVals[-2+yyTop]), ((int)yyVals[-9+yyTop])); }
  break;
case 11:
#line 158 "Parser.jay"
  { CompilerOutput.Error("declaration of new channels must be `new ID:<schema>'", lexer.location); }
  break;
case 12:
#line 161 "Parser.jay"
  { yyVal = new Spawn(((IProcess)yyVals[-2+yyTop]), ((IProcess)yyVals[0+yyTop]), ((int)yyVals[-4+yyTop])); }
  break;
case 13:
#line 164 "Parser.jay"
  { yyVal = new MatchWith(((IExpression)yyVals[-4+yyTop]), ((ArrayList)yyVals[-1+yyTop]), ((int)yyVals[-5+yyTop])); }
  break;
case 14:
#line 172 "Parser.jay"
  { yyVal = new JoinPatternDefinitionProcess(((bool)yyVals[-1+yyTop]), ((ArrayList)yyVals[0+yyTop]), ((int)yyVals[-2+yyTop])); }
  break;
case 15:
#line 175 "Parser.jay"
  { yyVal = new Zero(((int)yyVals[0+yyTop]));}
  break;
case 16:
#line 178 "Parser.jay"
  { CompilerOutput.Error("Error in process expression", lexer.location); }
  break;
case 17:
#line 182 "Parser.jay"
  { yyVal = false; }
  break;
case 18:
#line 183 "Parser.jay"
  { yyVal = true; }
  break;
case 19:
#line 188 "Parser.jay"
  {
	  ArrayList rows = new ArrayList();
		rows.Add(((JoinPatternRow)yyVals[0+yyTop]));
		yyVal = rows;
	}
  break;
case 20:
#line 195 "Parser.jay"
  {
	  ArrayList rows = (ArrayList) ((ArrayList)yyVals[-2+yyTop]);
		rows.Add(((JoinPatternRow)yyVals[0+yyTop]));
		yyVal = rows;
	}
  break;
case 21:
#line 204 "Parser.jay"
  { yyVal = new JoinPatternRow(((ArrayList)yyVals[-2+yyTop]), ((IProcess)yyVals[0+yyTop]), ((IProcess)yyVals[0+yyTop]).Location); }
  break;
case 22:
#line 209 "Parser.jay"
  {
		ArrayList atoms = new ArrayList();
		atoms.Add(((JoinPatternAtom)yyVals[0+yyTop]));
		yyVal = atoms;
	}
  break;
case 23:
#line 215 "Parser.jay"
  {
		ArrayList atoms = (ArrayList) ((ArrayList)yyVals[-2+yyTop]);
		atoms.Add(((JoinPatternAtom)yyVals[0+yyTop]));
		yyVal = atoms;
	}
  break;
case 24:
#line 224 "Parser.jay"
  { yyVal = new JoinPatternAtom(((StringValue)yyVals[-3+yyTop]).Value, ((IPattern)yyVals[-1+yyTop]), ((StringValue)yyVals[-3+yyTop]).Location); }
  break;
case 25:
#line 229 "Parser.jay"
  {
		ArrayList decls = new ArrayList();
		decls.Add(((SchemaDec.Declaration)yyVals[0+yyTop]));
		yyVal = decls;
	}
  break;
case 26:
#line 236 "Parser.jay"
  {
		ArrayList decls = (ArrayList) ((ArrayList)yyVals[-2+yyTop]);
		decls.Add(((SchemaDec.Declaration)yyVals[0+yyTop]));
		yyVal = decls;
	}
  break;
case 27:
#line 245 "Parser.jay"
  { yyVal = new SchemaDec.Declaration(((StringValue)yyVals[-2+yyTop]).Value, ((IType)yyVals[0+yyTop]), ((StringValue)yyVals[-2+yyTop]).Location); }
  break;
case 28:
#line 250 "Parser.jay"
  {
	  yyVal = new ArrayList();
		((ArrayList) yyVal).Add(new MatchWith.Branch(((IPattern)yyVals[-2+yyTop]), ((IProcess)yyVals[0+yyTop])));
	}
  break;
case 29:
#line 256 "Parser.jay"
  {
		((ArrayList)yyVals[-4+yyTop]).Add(new MatchWith.Branch(((IPattern)yyVals[-2+yyTop]), ((IProcess)yyVals[0+yyTop])));
		yyVal = ((ArrayList)yyVals[-4+yyTop]);
	}
  break;
case 30:
#line 262 "Parser.jay"
  { CompilerOutput.Error("expected token |", lexer.location); }
  break;
case 31:
#line 286 "Parser.jay"
  { yyVal = new SchemaPattern(((IType)yyVals[0+yyTop]), lexer.location); }
  break;
case 32:
#line 289 "Parser.jay"
  { yyVal = new VarPattern(((StringValue)yyVals[-2+yyTop]).Value, ((IPattern)yyVals[0+yyTop]), ((StringValue)yyVals[-2+yyTop]).Location); }
  break;
case 33:
#line 292 "Parser.jay"
  {
		if ((((LabelsSet)yyVals[-3+yyTop]) is UnionLabel) && ((UnionLabel)((LabelsSet)yyVals[-3+yyTop])).labels.Count==0) {
			CompilerOutput.Error("emptyset of label in pattern",((int)yyVals[-2+yyTop]));
			yyVal = ((IPattern)yyVals[-1+yyTop]);
		}
		
		if ((((LabelsSet)yyVals[-3+yyTop]) is DifferenceLabel) && ((DifferenceLabel) ((LabelsSet)yyVals[-3+yyTop])).diffLabels.labels.Count == 0) {
			LabelsSet anyl = new AnyLabel();
			locationMap.Add(anyl, GetLocation(((LabelsSet)yyVals[-3+yyTop])));
			yyVal = new LabelPattern(anyl, ((IPattern)yyVals[-1+yyTop]), GetLocation(((LabelsSet)yyVals[-3+yyTop])));
		} else
			yyVal = new LabelPattern(((LabelsSet)yyVals[-3+yyTop]),((IPattern)yyVals[-1+yyTop]),GetLocation(((LabelsSet)yyVals[-3+yyTop])));
	}
  break;
case 34:
#line 307 "Parser.jay"
  {
		IPattern f;
		if ((((LabelsSet)yyVals[-5+yyTop]) is UnionLabel) && ((UnionLabel) ((LabelsSet)yyVals[-5+yyTop])).labels.Count == 0) {
			CompilerOutput.Error("emptyset of label in pattern",((int)yyVals[-4+yyTop]));
			f = ((IPattern)yyVals[-3+yyTop]);
		}

		if ((((LabelsSet)yyVals[-5+yyTop]) is DifferenceLabel) && ((DifferenceLabel) ((LabelsSet)yyVals[-5+yyTop])).diffLabels.labels.Count == 0) {
			LabelsSet anyl= new AnyLabel();
			f = new LabelPattern(anyl, ((IPattern)yyVals[-3+yyTop]), GetLocation(((LabelsSet)yyVals[-5+yyTop])));
			locationMap.Add(anyl, GetLocation(((LabelsSet)yyVals[-5+yyTop])));
		} else 
			f = new LabelPattern(((LabelsSet)yyVals[-5+yyTop]), ((IPattern)yyVals[-3+yyTop]), GetLocation(((LabelsSet)yyVals[-5+yyTop])));

		yyVal = new SequencePattern(f, ((IPattern)yyVals[0+yyTop]),GetLocation(((LabelsSet)yyVals[-5+yyTop])));
	}
  break;
case 35:
#line 326 "Parser.jay"
  {yyVal = ((IExpression)yyVals[-1+yyTop]);}
  break;
case 36:
#line 327 "Parser.jay"
  {yyVal = new VoidExp(((int)yyVals[0+yyTop]));}
  break;
case 37:
#line 328 "Parser.jay"
  {yyVal = new VariableExp(((StringValue)yyVals[0+yyTop]).Value, ((StringValue)yyVals[0+yyTop]).Location);}
  break;
case 38:
#line 329 "Parser.jay"
  {yyVal = new IntExp(((IntValue)yyVals[0+yyTop]).Value, ((IntValue)yyVals[0+yyTop]).Location);}
  break;
case 39:
#line 330 "Parser.jay"
  {yyVal = new IntExp(0, ((int)yyVals[0+yyTop]));}
  break;
case 40:
#line 331 "Parser.jay"
  {yyVal = new StringExp(((StringValue)yyVals[0+yyTop]).Value, ((StringValue)yyVals[0+yyTop]).Location);}
  break;
case 41:
#line 332 "Parser.jay"
  {yyVal = new LabelExp(((StringValue)yyVals[-3+yyTop]).Value, ((IExpression)yyVals[-1+yyTop]), ((StringValue)yyVals[-3+yyTop]).Location);}
  break;
case 42:
#line 333 "Parser.jay"
  {yyVal = new LabelExp(((StringValue)yyVals[-2+yyTop]).Value, new BoPi.Compiler.VoidExp(((int)yyVals[-1+yyTop])), ((StringValue)yyVals[-2+yyTop]).Location);}
  break;
case 43:
#line 334 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.PLUS, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 44:
#line 335 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.MINUS, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 45:
#line 336 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.TIMES, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 46:
#line 337 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.DIVIDE, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 47:
#line 338 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.EQEQ, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 48:
#line 339 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.NE, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 49:
#line 340 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.GT, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 50:
#line 341 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.GE, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 51:
#line 342 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.LT, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 52:
#line 343 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.LE, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 53:
#line 344 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.OR, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 54:
#line 345 "Parser.jay"
  {yyVal = new BinaryExp(((IExpression)yyVals[-2+yyTop]), BinaryOp.AND, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[-2+yyTop]).Location);}
  break;
case 55:
#line 346 "Parser.jay"
  {yyVal = new SequenceExp(new LabelExp(((StringValue)yyVals[-5+yyTop]).Value, ((IExpression)yyVals[-3+yyTop]), ((StringValue)yyVals[-5+yyTop]).Location), ((IExpression)yyVals[0+yyTop]), ((StringValue)yyVals[-5+yyTop]).Location);}
  break;
case 56:
#line 347 "Parser.jay"
  {yyVal = new SequenceExp(new LabelExp(((StringValue)yyVals[-4+yyTop]).Value, new BoPi.Compiler.VoidExp(((int)yyVals[-3+yyTop])), ((StringValue)yyVals[-4+yyTop]).Location), ((IExpression)yyVals[0+yyTop]), ((StringValue)yyVals[-4+yyTop]).Location);}
  break;
case 57:
#line 348 "Parser.jay"
  {yyVal = new UnaryExp(UnaryOp.NOT, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[0+yyTop]).Location);}
  break;
case 58:
#line 349 "Parser.jay"
  {yyVal = new UnaryExp(UnaryOp.UMINUS, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[0+yyTop]).Location);}
  break;
case 59:
#line 350 "Parser.jay"
  {yyVal = new UnaryExp(UnaryOp.REST, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[0+yyTop]).Location);}
  break;
case 60:
#line 351 "Parser.jay"
  {yyVal = new UnaryExp(UnaryOp.FIRST, ((IExpression)yyVals[0+yyTop]), ((IExpression)yyVals[0+yyTop]).Location);}
  break;
case 61:
#line 353 "Parser.jay"
  { CompilerOutput.Error("Error in expression", lexer.location); }
  break;
case 62:
#line 358 "Parser.jay"
  { 
		yyVal = new UnionLabel(((StringValue)yyVals[0+yyTop]).Value);
		locationMap.Add(yyVal, ((StringValue)yyVals[0+yyTop]).Location);
	}
  break;
case 63:
#line 364 "Parser.jay"
  {
		yyVal = new AnyLabel();
		locationMap.Add(yyVal, ((int)yyVals[0+yyTop]));
	}
  break;
case 64:
#line 370 "Parser.jay"
  { yyVal = LabelsSet.NormalizeUnion(((LabelsSet)yyVals[-2+yyTop]), ((LabelsSet)yyVals[0+yyTop])); }
  break;
case 65:
#line 373 "Parser.jay"
  { yyVal = LabelsSet.NormalizeDifference(((LabelsSet)yyVals[-2+yyTop]), ((LabelsSet)yyVals[0+yyTop])); }
  break;
case 66:
#line 376 "Parser.jay"
  { yyVal = ((LabelsSet)yyVals[-1+yyTop]); }
  break;
case 67:
#line 379 "Parser.jay"
  {yyVal= new Types.Void();}
  break;
case 68:
#line 380 "Parser.jay"
  {yyVal= new Types.Void();}
  break;
case 69:
#line 381 "Parser.jay"
  {yyVal= ((IType)yyVals[-1+yyTop]);}
  break;
case 70:
#line 382 "Parser.jay"
  {yyVal= new Union (((IType)yyVals[-2+yyTop]),((IType)yyVals[0+yyTop]));}
  break;
case 71:
#line 383 "Parser.jay"
  {yyVal= new ConstantTypeName(((StringValue)yyVals[0+yyTop]).Value);}
  break;
case 72:
#line 384 "Parser.jay"
  {if ((((LabelsSet)yyVals[-3+yyTop]) is UnionLabel) && ((UnionLabel)((LabelsSet)yyVals[-3+yyTop])).labels.Count==0)
					{
						CompilerOutput.Error("emptyset of label",((int)yyVals[-2+yyTop]));
						yyVal=((IType)yyVals[-1+yyTop]);
					}
					if ((((LabelsSet)yyVals[-3+yyTop]) is DifferenceLabel) &&
            ((DifferenceLabel)((LabelsSet)yyVals[-3+yyTop])).diffLabels.labels.Count==0){
              LabelsSet anyl= new AnyLabel();
              locationMap.Add(anyl,GetLocation(((LabelsSet)yyVals[-3+yyTop])));
					    yyVal= new Labelled(new AnyLabel(),((IType)yyVals[-1+yyTop]));
              locationMap.Add(yyVal,GetLocation(((LabelsSet)yyVals[-3+yyTop])));
          }
					else
					    yyVal= new Labelled(((LabelsSet)yyVals[-3+yyTop]),((IType)yyVals[-1+yyTop]));}
  break;
case 73:
#line 398 "Parser.jay"
  {if ((((LabelsSet)yyVals[-2+yyTop]) is UnionLabel) && ((UnionLabel)((LabelsSet)yyVals[-2+yyTop])).labels.Count==0)
					{
						CompilerOutput.Error("emptyset of label",((int)yyVals[-1+yyTop]));
						yyVal=new Types.Void();
					}
					if ((((LabelsSet)yyVals[-2+yyTop]) is DifferenceLabel) &&
            ((DifferenceLabel)((LabelsSet)yyVals[-2+yyTop])).diffLabels.labels.Count==0){
              LabelsSet anyl= new AnyLabel();
              locationMap.Add(anyl,GetLocation(((LabelsSet)yyVals[-2+yyTop])));
					    yyVal= new Labelled(anyl,new Types.Void());
              locationMap.Add(yyVal,GetLocation(((LabelsSet)yyVals[-2+yyTop])));
          }
					else
					    yyVal= new Labelled(((LabelsSet)yyVals[-2+yyTop]),new Types.Void());}
  break;
case 74:
#line 413 "Parser.jay"
  {if ((((LabelsSet)yyVals[-5+yyTop]) is UnionLabel) && ((UnionLabel)((LabelsSet)yyVals[-5+yyTop])).labels.Count==0)
					{
						CompilerOutput.Error("emptyset of label",((int)yyVals[-4+yyTop]));
						yyVal=new Sequence(new Labelled(((LabelsSet)yyVals[-5+yyTop]),((IType)yyVals[-3+yyTop])),((IType)yyVals[0+yyTop]));
					}
					if ((((LabelsSet)yyVals[-5+yyTop]) is DifferenceLabel) &&
            ((DifferenceLabel)((LabelsSet)yyVals[-5+yyTop])).diffLabels.labels.Count==0){
              LabelsSet anyl= new AnyLabel();
              locationMap.Add(yyVal,GetLocation(((LabelsSet)yyVals[-5+yyTop])));
					    yyVal= new Sequence(new Labelled(anyl,((IType)yyVals[-3+yyTop])),((IType)yyVals[0+yyTop]));
          }
					else
					    yyVal= new Sequence(new Labelled(((LabelsSet)yyVals[-5+yyTop]),((IType)yyVals[-3+yyTop])),((IType)yyVals[0+yyTop]));}
  break;
case 75:
#line 426 "Parser.jay"
  {if ((((LabelsSet)yyVals[-4+yyTop]) is UnionLabel) && ((UnionLabel)((LabelsSet)yyVals[-4+yyTop])).labels.Count==0)
					{
						CompilerOutput.Error("emptyset of label",((int)yyVals[-3+yyTop]));
						yyVal=new Sequence(new Labelled(((LabelsSet)yyVals[-4+yyTop]), new Types.Void()),((IType)yyVals[0+yyTop]));
					}
					if ((((LabelsSet)yyVals[-4+yyTop]) is DifferenceLabel) && ((DifferenceLabel)((LabelsSet)yyVals[-4+yyTop])).diffLabels.labels.Count==0)
					    yyVal= new Sequence(new Labelled(new AnyLabel(),new Types.Void()),((IType)yyVals[0+yyTop]));
					else
					    yyVal= new Sequence(new Labelled(((LabelsSet)yyVals[-4+yyTop]),new Types.Void()),((IType)yyVals[0+yyTop]));}
  break;
case 76:
#line 435 "Parser.jay"
  {yyVal= new Chan(((IType)yyVals[-2+yyTop]),((int)yyVals[0+yyTop]));}
  break;
case 77:
#line 436 "Parser.jay"
  {yyVal= ((IType)yyVals[0+yyTop]);}
  break;
case 78:
#line 438 "Parser.jay"
  { CompilerOutput.Error("Error in schema expression", lexer.location); }
  break;
case 79:
#line 441 "Parser.jay"
  {yyVal=Chan.CAPABILITY.IN;}
  break;
case 80:
#line 442 "Parser.jay"
  {yyVal=Chan.CAPABILITY.OUT;}
  break;
case 81:
#line 443 "Parser.jay"
  {yyVal=Chan.CAPABILITY.INOUT;}
  break;
case 82:
#line 444 "Parser.jay"
  { CompilerOutput.Error("wrong capability", lexer.location);}
  break;
case 83:
#line 447 "Parser.jay"
  {yyVal= new IntType();}
  break;
case 84:
#line 448 "Parser.jay"
  {yyVal= new StringType();}
  break;
case 85:
#line 449 "Parser.jay"
  {yyVal= new IntLiteral(0);}
  break;
case 86:
#line 450 "Parser.jay"
  {yyVal= new IntLiteral(((IntValue)yyVals[0+yyTop]).Value);}
  break;
case 87:
#line 451 "Parser.jay"
  {yyVal= new IntLiteral(-((IntValue)yyVals[0+yyTop]).Value);}
  break;
case 88:
#line 452 "Parser.jay"
  {yyVal= new StringLiteral(((StringValue)yyVals[0+yyTop]).Value);}
  break;
#line default
        }
        yyTop -= yyLen[yyN];
        yyState = yyStates[yyTop];
        int yyM = yyLhs[yyN];
        if (yyState == 0 && yyM == 0) {
//t          if (debug != null) debug.shift(0, yyFinal);
          yyState = yyFinal;
          if (yyToken < 0) {
            yyToken = yyLex.advance() ? yyLex.token() : 0;
//t            if (debug != null)
//t               debug.lex(yyState, yyToken,yyname(yyToken), yyLex.value());
          }
          if (yyToken == 0) {
//t            if (debug != null) debug.accept(yyVal);
            return yyVal;
          }
          goto yyLoop;
        }
        if (((yyN = yyGindex[yyM]) != 0) && ((yyN += yyState) >= 0)
            && (yyN < yyTable.Length) && (yyCheck[yyN] == yyState))
          yyState = yyTable[yyN];
        else
          yyState = yyDgoto[yyM];
//t        if (debug != null) debug.shift(yyStates[yyTop], yyState);
	 goto yyLoop;
      }
    }
  }

   static  short [] yyLhs  = {              -1,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    1,    1,    9,    9,
   12,   10,   10,   13,   11,   11,   14,    2,    2,    2,
    3,    3,    3,    3,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
    4,    7,    7,    7,    7,    7,    5,    5,    5,    5,
    5,    5,    5,    5,    5,    5,    5,    5,    8,    8,
    8,    8,    6,    6,    6,    6,    6,    6,
  };
   static  short [] yyLen = {           2,
    4,    7,    6,    4,    9,   10,    8,    9,    9,   10,
    4,    5,    6,    3,    1,    1,    0,    1,    1,    3,
    3,    1,    3,    4,    1,    3,    3,    3,    5,    4,
    1,    3,    4,    6,    3,    1,    1,    1,    1,    1,
    4,    3,    3,    3,    3,    3,    3,    3,    3,    3,
    3,    3,    3,    3,    6,    5,    2,    2,    2,    2,
    1,    1,    1,    3,    3,    3,    2,    1,    3,    3,
    1,    4,    3,    6,    5,    4,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    2,    1,
  };
   static  short [] yyDefRed = {            0,
   16,    0,    0,    0,    0,   15,    0,    0,    0,    0,
    0,    0,   61,    0,    0,   36,   40,   38,   39,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   25,    0,
    0,    0,    0,   18,    0,    0,    0,   58,    0,   57,
   60,   59,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   78,    0,    0,    0,    0,   63,   83,   84,   68,
   88,   86,   85,    0,    0,    0,   77,    0,    0,    0,
    0,    0,    0,   19,   22,    0,   35,    0,    0,    0,
    0,    0,    0,   45,   46,    0,    0,    0,    0,    0,
    0,    0,   11,    0,    0,    0,    0,    4,   26,    1,
   62,    0,   67,    0,   87,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,   12,    0,
    0,    0,    0,    0,    0,   66,   69,    0,   32,    0,
    0,    0,    0,    0,   65,   64,    0,    0,    0,   20,
   21,   23,    0,    0,   13,    0,    0,    0,    0,   82,
   79,   80,   81,   76,    3,    0,    0,    0,    0,    0,
   24,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    2,    0,    0,   30,    7,    0,    0,    0,   34,    0,
    0,    0,   29,    8,    5,    0,    9,    0,    6,   10,
  };
  protected static  short [] yyDgoto  = {            11,
   35,  132,   75,   24,   76,   77,  107,  164,   82,   83,
   28,   84,   85,   29,
  };
  protected static  short [] yySindex = {         -186,
    0, -160,  -88, -250, -293,    0, -190, -133, -202, -154,
    0, -186,    0,  -88,  -88,    0,    0,    0,    0, -134,
  -88,  -88,  -88,  182, -155, -132, -127, -285,    0,  -88,
  194, -120, -117,    0, -141,  -99,  230,    0, -104,    0,
    0,    0,  -88,  -88,  -88,  -88,  -88,  -88,  -88,  -88,
  -88,  -88, -103,  -88,  -88, -186, -106,  242, -186, -293,
  278,    0, -259,   78, -121,  242,    0,    0,    0,    0,
    0,    0,    0, -100,  -90,  -92,    0, -173,  194,  -97,
  -79, -112, -135,    0,    0, -186,    0,  -95,  304,  196,
  196, -152, -152,    0,    0,  196,  196,  196,  196,  194,
  403,  403,    0,  242,    0,  -92, -166,    0,    0,    0,
    0, -254,    0, -192,    0, -269,  194,  -72,  242,  114,
 -259, -259,  -69,  242,  194, -141, -186, -141,    0,  -88,
  -83, -256,  -54, -268,  150,    0,    0,   -8,    0, -186,
  -92,  -77,  -62, -234,    0,    0,  -60, -165,  -53,    0,
    0,    0,  367,  -88,    0,  194, -186, -246, -234,    0,
    0,    0,    0,    0,    0,  242,  -68,  -63, -186,   -8,
    0,  367,  -34,  -28, -186,  -58, -219,  -92,  194,  242,
    0, -275, -186,    0,    0, -186, -186,  -56,    0,  -92,
 -186,  -55,    0,    0,    0, -186,    0, -186,    0,    0,
  };
  protected static  short [] yyRindex = {            0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  -71,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    6,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  385,    0, -230,    0,    0,    0,    0,
    0,    1,    0,    0,    0,    0,    0,   32,    0, -226,
 -113,  330,  356,    0,    0,  -27,   25,   55,   87,    0,
  127,  133,    0,    0, -137, -220,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
   58,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 -109,  -46,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, -249,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0, -150,  -42,    0,    0,
    0, -188,    0, -242,    0,    0,    0,  -36,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  -20,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  };
  protected static  short [] yyGindex = {          -12,
    0,    0,  -31,   10,  -37,    0,  -29,  -73,    0,    0,
    0,  106,  113,  183,
  };
  protected static  short [] yyTable = {            36,
   14,   78,  119,  119,   63,   25,   59,   27,  155,  160,
  136,  138,  158,  161,  162,  163,  191,   56,   60,   56,
  106,  121,   28,   37,   38,   67,  114,   31,  116,  122,
   40,   41,   42,  112,  168,  192,   31,  119,   31,   61,
   47,  111,   47,  103,   56,  175,  108,  123,   89,   78,
   26,  156,   90,   91,   92,   93,   94,   95,   96,   97,
   98,   99,  176,  101,  102,   28,  134,   47,  133,    1,
   78,   27,  187,  129,  137,   30,   47,   47,   55,  119,
   55,  141,  144,   27,  177,  139,  148,   78,  143,  188,
   78,  145,  146,  149,  120,   78,  182,  159,   33,    2,
    3,  135,  121,   12,    4,   55,  119,   33,    5,  121,
  122,    6,    7,    8,  151,  170,   33,  122,   33,   34,
   71,   47,   48,    9,  173,   10,   78,  165,  178,   71,
   62,   71,   31,   39,   71,  127,   56,   57,   62,  153,
   32,  128,  190,   71,  174,   79,   62,  189,   70,   78,
   58,   13,   80,   48,   71,   48,  181,   70,   81,   70,
  100,   14,  185,  172,   88,   86,   71,   13,   15,  117,
  193,   70,  104,  194,  195,  115,  118,   14,  197,  119,
   48,  124,   70,  199,   15,  200,  125,  130,   16,   48,
   48,   17,   18,   19,   70,  126,   20,  147,  140,  154,
   21,   22,   23,  157,   16,  166,  167,   17,   18,   19,
  169,   73,   20,  171,  179,   72,   21,   22,   23,  180,
   73,   75,   73,  183,   72,   73,   72,  184,   17,   72,
   75,  150,   75,  186,   73,  196,  198,   74,   72,   51,
  152,   51,  109,    0,   75,   73,   74,  160,   74,   72,
    0,  161,  162,  163,    0,   75,   14,   73,    0,    0,
   74,   72,    0,    0,   37,   14,   51,   75,   37,    0,
    0,   74,   37,    0,   37,   51,   51,   37,   37,   37,
   37,    0,    0,   74,   37,   37,   37,   37,    0,    0,
   42,   52,    0,   52,   42,    0,    0,    0,   42,   37,
   42,    0,    0,   42,   42,   42,   42,    0,   37,   37,
   42,   42,   42,   42,    0,    0,   41,    0,   52,    0,
   41,   49,    0,   49,   41,   42,   41,   52,   52,   41,
   41,   41,   41,   62,   42,   42,   41,   41,   41,   41,
    0,   63,    0,   64,  113,    0,    0,    0,   49,    0,
   65,   41,    0,   50,    0,   50,   66,   49,   49,    0,
   41,   41,   67,    0,    0,    0,   68,   69,    0,   62,
   70,    0,    0,   71,   72,   73,    0,   63,  105,   64,
   50,    0,  142,    0,    0,    0,   65,    0,    0,   50,
   50,    0,   66,   53,    0,   53,    0,    0,   67,   54,
    0,   54,   68,   69,    0,   62,   70,    0,    0,   71,
   72,   73,    0,   63,   74,   64,    0,    0,  142,    0,
   53,    0,   65,    0,    0,    0,   54,    0,   66,   53,
   53,    0,    0,    0,   67,   54,   54,    0,   68,   69,
   43,    0,   70,    0,   44,   71,   72,   73,    0,   62,
  105,    0,    0,   45,   46,   47,   48,   63,    0,   64,
   49,   50,   51,   52,    0,    0,   65,   45,   46,   47,
   48,    0,   66,    0,    0,   53,    0,    0,   67,    0,
    0,    0,   68,   69,   54,   55,   70,    0,   43,   71,
   72,   73,   44,    0,   74,    0,   87,   62,    0,    0,
    0,   45,   46,   47,   48,   63,    0,   64,   49,   50,
   51,   52,    0,    0,   65,    0,    0,    0,    0,    0,
   66,    0,    0,    0,    0,    0,   67,    0,    0,    0,
   68,   69,   54,   55,   70,    0,   43,   71,   72,   73,
   44,    0,  105,    0,  110,    0,    0,    0,    0,   45,
   46,   47,   48,    0,    0,    0,   49,   50,   51,   52,
    0,    0,   43,    0,    0,    0,   44,    0,    0,    0,
    0,    0,  131,    0,    0,   45,   46,   47,   48,    0,
   54,   55,   49,   50,   51,   52,    0,    0,   43,    0,
    0,    0,   43,    0,    0,    0,   43,    0,   43,    0,
    0,   43,   43,    0,    0,    0,   54,   55,   43,   43,
   43,   43,    0,    0,   44,    0,    0,    0,   44,    0,
    0,    0,   44,   43,   44,   43,    0,   44,   44,   44,
    0,    0,   43,   43,   44,   44,   44,   44,   45,   46,
   47,   48,   71,    0,    0,   49,   50,   51,   52,   44,
    0,   71,   62,   71,    0,    0,   71,    0,   44,   44,
   62,   43,    0,    0,    0,   44,    0,    0,   62,   54,
   55,    0,    0,    0,   45,   46,   47,   48,    0,    0,
    0,   49,   50,   51,   52,
  };
  protected static  short [] yyCheck = {            12,
    0,   31,  272,  272,  264,  256,  292,  301,  265,  256,
  265,  281,  281,  260,  261,  262,  292,  267,  304,  269,
   58,  276,  265,   14,   15,  285,   64,  258,   66,  284,
   21,   22,   23,   63,  269,  311,  267,  272,  269,   30,
  267,  301,  269,   56,  294,  292,   59,   79,   39,   79,
  301,  308,   43,   44,   45,   46,   47,   48,   49,   50,
   51,   52,  309,   54,   55,  308,  104,  294,  100,  256,
  100,  292,  292,   86,  267,  266,  303,  304,  267,  272,
  269,  119,  120,  304,  158,  117,  124,  117,  120,  309,
  120,  121,  122,  125,  268,  125,  170,  135,  301,  286,
  287,  268,  276,  264,  291,  294,  272,  258,  295,  276,
  284,  298,  299,  300,  127,  281,  267,  284,  269,  274,
  258,  274,  275,  310,  156,  312,  156,  140,  166,  267,
  268,  269,  266,  268,  272,  271,  292,  270,  276,  130,
  274,  277,  180,  281,  157,  266,  284,  179,  258,  179,
  278,  256,  270,  267,  292,  269,  169,  267,  300,  269,
  264,  266,  175,  154,  269,  265,  304,  256,  273,  270,
  183,  281,  279,  186,  187,  297,  267,  266,  191,  272,
  294,  279,  292,  196,  273,  198,  266,  283,  293,  303,
  304,  296,  297,  298,  304,  308,  301,  267,  271,  283,
  305,  306,  307,  258,  293,  283,  269,  296,  297,  298,
  271,  258,  301,  267,  283,  258,  305,  306,  307,  283,
  267,  258,  269,  258,  267,  272,  269,  256,  300,  272,
  267,  126,  269,  292,  281,  292,  292,  258,  281,  267,
  128,  269,   60,   -1,  281,  292,  267,  256,  269,  292,
   -1,  260,  261,  262,   -1,  292,  256,  304,   -1,   -1,
  281,  304,   -1,   -1,  259,  265,  294,  304,  263,   -1,
   -1,  292,  267,   -1,  269,  303,  304,  272,  273,  274,
  275,   -1,   -1,  304,  279,  280,  281,  282,   -1,   -1,
  259,  267,   -1,  269,  263,   -1,   -1,   -1,  267,  294,
  269,   -1,   -1,  272,  273,  274,  275,   -1,  303,  304,
  279,  280,  281,  282,   -1,   -1,  259,   -1,  294,   -1,
  263,  267,   -1,  269,  267,  294,  269,  303,  304,  272,
  273,  274,  275,  256,  303,  304,  279,  280,  281,  282,
   -1,  264,   -1,  266,  267,   -1,   -1,   -1,  294,   -1,
  273,  294,   -1,  267,   -1,  269,  279,  303,  304,   -1,
  303,  304,  285,   -1,   -1,   -1,  289,  290,   -1,  256,
  293,   -1,   -1,  296,  297,  298,   -1,  264,  301,  266,
  294,   -1,  269,   -1,   -1,   -1,  273,   -1,   -1,  303,
  304,   -1,  279,  267,   -1,  269,   -1,   -1,  285,  267,
   -1,  269,  289,  290,   -1,  256,  293,   -1,   -1,  296,
  297,  298,   -1,  264,  301,  266,   -1,   -1,  269,   -1,
  294,   -1,  273,   -1,   -1,   -1,  294,   -1,  279,  303,
  304,   -1,   -1,   -1,  285,  303,  304,   -1,  289,  290,
  259,   -1,  293,   -1,  263,  296,  297,  298,   -1,  256,
  301,   -1,   -1,  272,  273,  274,  275,  264,   -1,  266,
  279,  280,  281,  282,   -1,   -1,  273,  272,  273,  274,
  275,   -1,  279,   -1,   -1,  294,   -1,   -1,  285,   -1,
   -1,   -1,  289,  290,  303,  304,  293,   -1,  259,  296,
  297,  298,  263,   -1,  301,   -1,  267,  256,   -1,   -1,
   -1,  272,  273,  274,  275,  264,   -1,  266,  279,  280,
  281,  282,   -1,   -1,  273,   -1,   -1,   -1,   -1,   -1,
  279,   -1,   -1,   -1,   -1,   -1,  285,   -1,   -1,   -1,
  289,  290,  303,  304,  293,   -1,  259,  296,  297,  298,
  263,   -1,  301,   -1,  267,   -1,   -1,   -1,   -1,  272,
  273,  274,  275,   -1,   -1,   -1,  279,  280,  281,  282,
   -1,   -1,  259,   -1,   -1,   -1,  263,   -1,   -1,   -1,
   -1,   -1,  269,   -1,   -1,  272,  273,  274,  275,   -1,
  303,  304,  279,  280,  281,  282,   -1,   -1,  259,   -1,
   -1,   -1,  263,   -1,   -1,   -1,  267,   -1,  269,   -1,
   -1,  272,  273,   -1,   -1,   -1,  303,  304,  279,  280,
  281,  282,   -1,   -1,  259,   -1,   -1,   -1,  263,   -1,
   -1,   -1,  267,  294,  269,  259,   -1,  272,  273,  263,
   -1,   -1,  303,  304,  279,  280,  281,  282,  272,  273,
  274,  275,  258,   -1,   -1,  279,  280,  281,  282,  294,
   -1,  267,  268,  269,   -1,   -1,  272,   -1,  303,  304,
  276,  259,   -1,   -1,   -1,  263,   -1,   -1,  284,  303,
  304,   -1,   -1,   -1,  272,  273,  274,  275,   -1,   -1,
   -1,  279,  280,  281,  282,
  };

#line 456 "Parser.jay"

    public Object parse (string file)
    {
        locationMap = new Hashtable();
        lexer = new Lexer(file);
        return yyparse (lexer);
    }
    public int GetLocation(Object obj)
    {
      return (int)locationMap[obj];
    }
}
#line default
namespace yydebug {
        using System;
	 internal interface yyDebug {
		 void push (int state, Object value);
		 void lex (int state, int token, string name, Object value);
		 void shift (int from, int to, int errorFlag);
		 void pop (int state);
		 void discard (int state, int token, string name, Object value);
		 void reduce (int from, int to, int rule, string text, int len);
		 void shift (int from, int to);
		 void accept (Object value);
		 void error (string message);
		 void reject ();
	 }
	 
	 class yyDebugSimple : yyDebug {
		 void println (string s){
			 Console.Error.WriteLine (s);
		 }
		 
		 public void push (int state, Object value) {
			 println ("push\tstate "+state+"\tvalue "+value);
		 }
		 
		 public void lex (int state, int token, string name, Object value) {
			 println("lex\tstate "+state+"\treading "+name+"\tvalue "+value);
		 }
		 
		 public void shift (int from, int to, int errorFlag) {
			 switch (errorFlag) {
			 default:				// normally
				 println("shift\tfrom state "+from+" to "+to);
				 break;
			 case 0: case 1: case 2:		// in error recovery
				 println("shift\tfrom state "+from+" to "+to
					     +"\t"+errorFlag+" left to recover");
				 break;
			 case 3:				// normally
				 println("shift\tfrom state "+from+" to "+to+"\ton error");
				 break;
			 }
		 }
		 
		 public void pop (int state) {
			 println("pop\tstate "+state+"\ton error");
		 }
		 
		 public void discard (int state, int token, string name, Object value) {
			 println("discard\tstate "+state+"\ttoken "+name+"\tvalue "+value);
		 }
		 
		 public void reduce (int from, int to, int rule, string text, int len) {
			 println("reduce\tstate "+from+"\tuncover "+to
				     +"\trule ("+rule+") "+text);
		 }
		 
		 public void shift (int from, int to) {
			 println("goto\tfrom state "+from+" to "+to);
		 }
		 
		 public void accept (Object value) {
			 println("accept\tvalue "+value);
		 }
		 
		 public void error (string message) {
			 println("error\t"+message);
		 }
		 
		 public void reject () {
			 println("reject");
		 }
		 
	 }
}
// %token constants
 class Token {
  public const int ERROR = 257;
  public const int ARROW = 258;
  public const int EQEQ = 259;
  public const int I = 260;
  public const int O = 261;
  public const int IO = 262;
  public const int NE = 263;
  public const int LBRACE = 264;
  public const int RBRACE = 265;
  public const int LPAREN = 266;
  public const int RPAREN = 267;
  public const int LBRACK = 268;
  public const int RBRACK = 269;
  public const int COLON = 270;
  public const int DOT = 271;
  public const int PLUS = 272;
  public const int MINUS = 273;
  public const int TIMES = 274;
  public const int DIVIDE = 275;
  public const int BACKSLASH = 276;
  public const int AMPERSAND = 277;
  public const int EQ = 278;
  public const int LT = 279;
  public const int LE = 280;
  public const int GT = 281;
  public const int GE = 282;
  public const int COMMA = 283;
  public const int SHARP = 284;
  public const int TILDE = 285;
  public const int SPAWN = 286;
  public const int MATCH = 287;
  public const int SELECT = 288;
  public const int INT = 289;
  public const int STRING = 290;
  public const int NEW = 291;
  public const int IN = 292;
  public const int VOID = 293;
  public const int WITH = 294;
  public const int SCHEMADEF = 295;
  public const int TEXT = 296;
  public const int NUMBER = 297;
  public const int ZERO = 298;
  public const int CHANOUT = 299;
  public const int CHANIN = 300;
  public const int ID = 301;
  public const int EXP = 302;
  public const int OR = 303;
  public const int AND = 304;
  public const int NOT = 305;
  public const int FIRST = 306;
  public const int REST = 307;
  public const int ORMATCH = 308;
  public const int LOCATION = 309;
  public const int IMPORT = 310;
  public const int IMPORTBODY = 311;
  public const int JOIN_KW = 312;
  public const int Selectdec = 313;
  public const int Declaration = 314;
  public const int ORPLUS = 315;
  public const int UMINUS = 316;
  public const int yyErrorCode = 256;
 }
 namespace yyParser {
  using System;
  /** thrown for irrecoverable syntax errors and stack overflow.
    */
  internal class yyException : System.Exception {
    public yyException (string message) : base (message) {
    }
  }

  /** must be implemented by a scanner object to supply input to the parser.
    */
  internal interface yyInput {
    /** move on to next token.
        @return false if positioned beyond tokens.
        @throws IOException on input error.
      */
    bool advance (); // throws java.io.IOException;
    /** classifies current token.
        Should not be called if advance() returned false.
        @return current %token or single character.
      */
    int token ();
    /** associated with current token.
        Should not be called if advance() returned false.
        @return value for token().
      */
    Object value ();
  }
 }
} // close outermost namespace, that MUST HAVE BEEN opened in the prolog
